@@ -159,8 +159,10 @@ class UnitTypeData:
         return self._game_data.abilities[self._proto.ability_id]
 
     @property
-    def footprint_radius(self) -> float:
+    def footprint_radius(self) -> Optional[float]:
         """ See unit.py footprint_radius """
+        if self.creation_ability is None:
+            return None
         return self.creation_ability._proto.footprint_radius
 
     @property
@@ -283,6 +285,11 @@ class UpgradeData:
 
 
 class Cost:
+    """
+    The cost of an action, a structure, a unit or a research upgrade.
+    The time is given in frames (22.4 frames per game second).
+    """
+
     def __init__(self, minerals: int, vespene: int, time: float = None):
         """
         :param minerals:
